package org.implab.gradle.mercurial.tasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.gradle.api.DefaultTask;
import org.gradle.api.model.ObjectFactory;
import org.gradle.api.provider.ListProperty;
import org.gradle.api.provider.Property;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.Utils;
import org.implab.gradle.mercurial.MercurialExtension;

import groovy.transform.Internal;

public class HgBookmark extends DefaultTask {

    private final Property<String> cliCmd;

    private final Property<Boolean> inactive;

    private final Property<Boolean> force;

    private final ListProperty<String> bookmarks;

    private String commandOutput;

    protected MercurialExtension getMercurialExtension() {
        return getProject().getExtensions().getByType(MercurialExtension.class);
    }

    @Inject
    public HgBookmark(ObjectFactory factory) {
        cliCmd = factory.property(String.class);
        inactive = factory.property(Boolean.class);
        force = factory.property(Boolean.class);
        bookmarks = factory.listProperty(String.class);
        cliCmd.convention(getMercurialExtension().getHgCmd());

        onlyIf(t -> !getBookmarks().get().isEmpty());
    }

    @Internal
    public Property<Boolean> getInactive() {
        return inactive;
    }

    @Internal
    public Property<Boolean> getForce() {
        return force;
    }

    @Internal
    public String getCommandOutput() {
        return commandOutput;
    }

    @Internal
    public ListProperty<String> getBookmarks() {
        return bookmarks;
    }

    @TaskAction
    public void Run() throws IOException, InterruptedException, Exception {
        List<String> commandLine = new ArrayList<>();

        commandLine.add(cliCmd.get());
        commandLine.add("bookmark");
        if(inactive.isPresent())
            commandLine.add("-i");
        if(force.isPresent())
            commandLine.add("-f");

        commandLine.addAll(bookmarks.get());

        getLogger().info("Starting: {}", commandLine);

        commandOutput = Utils.execCmd(commandLine, getMercurialExtension().getOutputCharset().name());
    }

}
